#!/bin/sh
#
# Return values of this script:
# 0   - Success
# 1   - Error


# Print an error message and exit
function errmsg () {
  echo "ERROR:" $@
  exit 1
}

EXEDIR=`pwd`

numarg=$#
if [ $numarg -lt 2 ]; then
  echo "ERROR: Incorrect number of arguments ($numarg)!"
  echo "Usage: <job> <input_arg> <SE> <lfc_addr>"
  exit 1
fi

# Executable
EXEC=$1
# Input data
ARG=$2
OUTPUT="job_${ARG}.out"
# Storage Element
SE="se.ui.savba.sk"
[ -n "$3" ] && SE=$3
# LFC Address 
LFCADDR="/grid/esr/sipka/tutor"
[ -n "$4" ] && LFCADDR=$4

echo `basename $0`" $1 $2 $3 $4"
echo "Arguments:"
echo "  EXEC=    $EXEC"
echo "  ARG=     $ARG"
echo "  SE=      $SE"
echo "  LFCADDR= $LFCADDR"
echo ""
echo "Environment Variables:"
echo "  LCG_GFAL_VO=      $LCG_GFAL_VO"
echo "  LFC_HOST=         $LFC_HOST"
echo "  LCG_CATALOG_TYPE= $LCG_CATALOG_TYPE"
echo "  LCG_GFAL_INFOSYS= $LCG_GFAL_INFOSYS"
echo ""

echo "Current dir: ${EXEDIR}"
ls -l
[ -e "$EXEC" ] || errmsg "File $EXEC not available!"
chmod +x $EXEC

APPLICATION=${EXEDIR}/${EXEC}
echo "==========================================="
echo "Host: "`hostname`
echo "Start "`date`
echo "${APPLICATION} ${OUTPUT}"
${APPLICATION} ${OUTPUT}
excode=$?
echo "End "`date`
echo "==========================================="

echo "Upload of ${OUTPUT} to $SE ($LFCADDR)"
[ -f "${OUTPUT}" ] || errmsg "File ${OUTPUT} not available!"
lcg-del -a lfn:${LFCADDR}/${OUTPUT} >/dev/null 2>&1
lcg-cr -d ${SE} file://${EXEDIR}/${OUTPUT} -l lfn:${LFCADDR}/${OUTPUT}
if [ $? -ne 0 ]; then
  errmsg "Upload of ${OUTPUT} to $SE ($LFCADDR) failed!"
fi

if [ $excode -ne 0 ]; then
  errmsg "${EXEC} failed!"
fi
echo "${EXEC} finished!"

exit 0

