#!/bin/sh
#
# Return values of this script:
# 0   - Success
# 1   - Error


# Print an error message and exit
function errmsg () {
  echo "ERROR:" $@
  exit 1
}

EXEDIR=`pwd`

numarg=$#
if [ $numarg -lt 3 ]; then
  echo "ERROR: Incorrect number of arguments ($numarg)!"
  echo "Usage: <job> <input_arg> <mpi_out> <SE> <lfc_addr>"
  exit 1
fi

# Executable
EXEC=$1
# Input data
ARG=$2
INPUT="job_${ARG}.out"
# MPI stdout+stderr
OARG=$3
# Storage Element
SE="se.ui.savba.sk"
[ -n "$4" ] && SE=$4
# LFC Address
LFCADDR="/grid/esr/sipka/tutor"
[ -n "$5" ] && LFCADDR=$5

echo `basename $0`" $1 $2 $3 $4 $5"
echo "Arguments:"
echo "  EXEC=    $EXEC"
echo "  ARG=     $ARG"
echo "  OARG=    $OARG"
echo "  SE=      $SE"
echo "  LFCADDR= $LFCADDR"
echo ""
echo "Environment Variables:"
echo "  LCG_GFAL_VO=      $LCG_GFAL_VO"
echo "  LFC_HOST=         $LFC_HOST"
echo "  LCG_CATALOG_TYPE= $LCG_CATALOG_TYPE"
echo "  LCG_GFAL_INFOSYS= $LCG_GFAL_INFOSYS"
echo ""

echo "Download of ${INPUT} from $SE ($LFCADDR)"
#lfc-ls -l ${LFCADDR}
lcg-cp lfn:${LFCADDR}/${INPUT} file://${EXEDIR}/${INPUT}
if [ $? -ne 0 ]; then
  errmsg "Download of ${INPUT} from $SE ($LFCADDR) failed!"
fi

echo "Current dir: ${EXEDIR}"
ls -l
[ -e "$EXEC" ] || errmsg "File $EXEC not available!"
chmod +x $EXEC
[ -e "${INPUT}" ] || errmsg "File ${INPUT} not available!"

#==================================================================
# Setup for mpiexec
#
MPI_FLAVOUR=OPENMPI
if [ "x$MPI_OPENMPI_PATH" == "x" ]; then
  echo "Path MPI_OPENMPI_PATH not set!"
# On ce2.ui.savba.sk
  MPI_OPENMPI_PATH="/opt/openmpi-1.3.2"
fi
echo "MPI_OPENMPI_PATH: $MPI_OPENMPI_PATH"
MPI_PREFIX="--prefix ${MPI_OPENMPI_PATH}"
MPI_EXEC=${MPI_OPENMPI_PATH}/bin/mpiexec
[ -e "${MPI_EXEC}" ] || errmsg "File ${MPI_EXEC} not available!"
#
if [ "x$PBS_NODEFILE" == "x" ]; then
   errmsg "Machinefile: PBS_NODEFILE not set!"
fi
echo "PBS_NODEFILE:"
cat $PBS_NODEFILE
NCPUS=`wc -l < $PBS_NODEFILE`
MPI_PARAMS="-np $NCPUS"
MPI_COMMAND="${MPI_EXEC} ${MPI_PREFIX} ${MPI_PARAMS}"
#
#==================================================================

APPLICATION=${EXEDIR}/${EXEC}
echo "==========================================="
echo "Host: "`hostname`
echo "Start "`date`
echo "${MPI_COMMAND} --output-filename ${OARG} ${APPLICATION} ${INPUT}"
${MPI_COMMAND} --output-filename ${OARG} ${APPLICATION} ${INPUT}
excode=$?
echo "End "`date`
echo "==========================================="
cat ${OARG}* >${OARG}
ls -l

#lfc-ls ${LFCADDR}/${INPUT} >/dev/null 2>&1
#if [ $? -eq 0 ]; then
#  lcg-del -a lfn:${LFCADDR}/${INPUT} >/dev/null 2>&1
#fi

if [ $excode -ne 0 ]; then
  errmsg "${EXEC} failed!"
fi
echo "${EXEC} finished!"

exit 0

