#!/bin/sh
#
# Return values of this script:
# 0   - Success
# 1   - Error


# Print an error message and exit
function errmsg () {
  echo "ERROR:" $@
  exit 1
}

EXEDIR=`pwd`

numarg=$#
if [ $numarg -lt 3 ]; then
  echo "ERROR: Incorrect number of arguments ($numarg)!"
  echo "Usage: <job> <input_arg> <transfer> <SE> <lfc_addr>"
  #transer: 0/1/2: nothing/download/upload"
  exit 1
fi

# Executable
EXEC=$1
# Input data
ARG=$2
INOUT="job_${ARG}.out"
# Transfer of Input/Output file from/to SE
TRANSF=$3
# Storage Element
SE="se.ui.savba.sk"
[ -n "$4" ] && SE=$4
# LFC Address
LFCADDR="/grid/esr/sipka/tutor"
[ -n "$5" ] && LFCADDR=$5

echo `basename $0`" $1 $2 $3 $4 $5"
echo "Arguments:"
echo "  EXEC=    $EXEC"
echo "  ARG=     $ARG"
echo "  TRANSF=  $TRANSF"
echo "  SE=      $SE"
echo "  LFCADDR= $LFCADDR"
echo ""
echo "Environment Variables:"
echo "  LCG_GFAL_VO=      $LCG_GFAL_VO"
echo "  LFC_HOST=         $LFC_HOST"
echo "  LCG_CATALOG_TYPE= $LCG_CATALOG_TYPE"
echo "  LCG_GFAL_INFOSYS= $LCG_GFAL_INFOSYS"
echo ""

echo "Current dir: ${EXEDIR}"

if [ $TRANSF -eq 1 ]; then
  echo "Download of ${INOUT} from $SE ($LFCADDR)"
  lcg-cp lfn:${LFCADDR}/${INOUT} file://${EXEDIR}/${INOUT}
  if [ $? -ne 0 ]; then
    lfc-ls -l ${LFCADDR}
    errmsg "Download of ${INOUT} from $SE ($LFCADDR) failed!"
  fi
fi

ls -l
[ -e "$EXEC" ] || errmsg "File $EXEC not available!"
chmod +x $EXEC
if [ $TRANSF -eq 1 ]; then
  [ -e "${INOUT}" ] || errmsg "File ${INOUT} not available!"
fi

APPLICATION=${EXEDIR}/${EXEC}
echo "==========================================="
echo "Host: "`hostname`
echo "Start "`date`
echo "${APPLICATION} ${INOUT}"
${APPLICATION} ${INOUT}
excode=$?
echo "End "`date`
echo "==========================================="

if [ $TRANSF -eq 2 ]; then
  [ -f "${INOUT}" ] || errmsg "File ${INOUT} not available!"
  echo "Upload of ${INOUT} to $SE ($LFCADDR)"
  lcg-del -a lfn:${LFCADDR}/${INOUT} >/dev/null 2>&1
  lcg-cr -d ${SE} file://${EXEDIR}/${INOUT} -l lfn:${LFCADDR}/${INOUT}
  if [ $? -ne 0 ]; then
    errmsg "Upload of ${INOUT} to $SE ($LFCADDR) failed!"
  fi
fi

if [ $excode -ne 0 ]; then
  errmsg "${EXEC} failed!"
fi
echo "${EXEC} finished!"

exit 0

