#!/bin/sh
#
# Return values of this script:
# 0   - Success
# 1   - Error


# Print an error message and exit
function errmsg () {
  echo "ERROR:" $@
  exit 1
}

EXEDIR=`pwd`

numarg=$#
if [ $numarg -lt 2 ]; then
  echo "ERROR: Incorrect number of arguments ($numarg)!"
  echo "Usage: <job> <input_arg> <SE> <lfc_addr>"
  exit 1
fi

# Executable
EXEC=$1
# Input data
ARG=$2
INPUT="job_${ARG}.out"
# Storage Element
SE="se.ui.savba.sk"
[ -n "$3" ] && SE=$3
# LFC Address
LFCADDR="/grid/esr/sipka/tutor"
[ -n "$4" ] && LFCADDR=$4

echo "Arguments:"
echo "  EXEC=    $EXEC"
echo "  ARG=     $ARG"
echo "  SE=      $SE"
echo "  LFCADDR= $LFCADDR"

echo "Environment Variables:"
echo "  LCG_GFAL_VO=      $LCG_GFAL_VO"
echo "  LFC_HOST=         $LFC_HOST"
echo "  LCG_CATALOG_TYPE= $LCG_CATALOG_TYPE"
echo "  LCG_GFAL_INFOSYS= $LCG_GFAL_INFOSYS"

echo "Download of ${INPUT} from $SE ($LFCADDR)"
lfc-ls -l ${LFCADDR}
lcg-cp lfn:${LFCADDR}/${INPUT} file://${EXEDIR}/${INPUT}
if [ $? -ne 0 ]; then
  errmsg "Download of ${INPUT} from $SE ($LFCADDR) failed!"
fi

echo "Current dir: ${EXEDIR}"
ls -l
[ -e "$EXEC" ] || errmsg "File $EXEC not available!"
chmod +x $EXEC
[ -e "${INPUT}" ] || errmsg "File ${INPUT} not available!"


APPLICATION=${EXEDIR}/${EXEC}
echo "==========================================="
echo "Host: "`hostname`
echo "Start "`date`
echo "${APPLICATION} ${INPUT}"
${APPLICATION} ${INPUT}
excode=$?
echo "End "`date`
echo "==========================================="

#lfc-ls ${LFCADDR}/${INPUT} >/dev/null 2>&1
#if [ $? -eq 0 ]; then
#  lcg-del -a lfn:${LFCADDR}/${INPUT} >/dev/null 2>&1
#fi

if [ $excode -ne 0 ]; then
  errmsg "${EXEC} failed!"
fi
echo "${EXEC} finished!"

exit 0

