      PROGRAM GAUSS
      ! Solving linear equation system  Ax = b
      ! A : Coefficient matrix with dimension (N,N+1)
      ! Right hand side vector of linear equations is stored
      !   in last (N+1)-th column of matrix A
      ! X : Vector of unknowns
      ! N : Number of linear equations

      IMPLICIT NONE
      
      INCLUDE 'mpif.h'

      INTEGER, PARAMETER :: Prec = SELECTED_REAL_KIND(15)
      ! SELECTED_REAL_KIND(15) = 8
      ! REAL(KIND=8) <=> DOUBLE PRECISION
      !INTEGER, PARAMETER :: N = 10
      INTEGER :: N
      !REAL(KIND=Prec), DIMENSION(N,N+1) :: A
      !REAL(KIND=Prec), DIMENSION(N) :: X
      !REAL(KIND=Prec), DIMENSION(N+1) :: temp
      !REAL(KIND=Prec) :: abspivot, mult
      !REAL(KIND=Prec), PARAMETER :: epsilon = 1E-13
      DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: A
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: X
      LOGICAL :: singular
      CHARACTER*15 :: fname
      CHARACTER*70 :: buffer
      INTEGER :: argc, xstat
      INTEGER :: stime(4), ftime(4), etime(4), values(8)
      CHARACTER*8 :: date    ! YYYYMMDD
      CHARACTER*10 :: xtime  ! HHMMSS.SSS
      CHARACTER*5 :: zone    ! HHMM
      INTEGER :: i, j, l
      INTEGER :: me, nprocs, ierr, root=0
      !INTEGER :: MPI_COMM_WORLD
      INTEGER :: status(MPI_STATUS_SIZE)

      ! Initialize MPI
      CALL MPI_INIT(ierr)
      CALL MPI_COMM_RANK(MPI_COMM_WORLD, me, ierr)
      CALL MPI_COMM_SIZE(MPI_COMM_WORLD, nprocs, ierr)

      ! Initialization of matrix
      ! Solution: X(1:N) = (/1.0,2.0,5.0,3.0,5.0,8.0,1.0,4.0,3.0,6.0/)
      ! A(1,1:N+1) = (/3.0,2.0,1.0,1.0,5.0,2.0,4.0,1.0,3.0,6.0, 109.0/)
      ! A(2,1:N+1) = (/1.0,2.0,2.0,1.0,4.0,3.0,4.0,5.0,3.0,1.0, 101.0/)
      ! A(3,1:N+1) = (/4.0,4.0,3.0,5.0,1.0,5.0,2.0,2.0,7.0,2.0, 130.0/)
      ! A(4,1:N+1) = (/2.0,3.0,4.0,2.0,1.0,4.0,1.0,3.0,1.0,3.0, 105.0/)
      ! A(5,1:N+1) = (/3.0,1.0,5.0,3.0,2.0,8.0,7.0,4.0,2.0,4.0, 166.0/)
      ! A(6,1:N+1) = (/1.0,9.0,8.0,1.0,3.0,1.0,5.0,1.0,1.0,1.0, 103.0/)
      ! A(7,1:N+1) = (/8.0,1.0,2.0,4.0,2.0,1.0,1.0,7.0,2.0,2.0, 97.0/)
      ! A(8,1:N+1) = (/7.0,1.0,1.0,2.0,2.0,3.0,9.0,4.0,5.0,5.0, 124.0/)
      ! A(9,1:N+1) = (/2.0,2.0,2.0,3.0,4.0,2.0,1.0,3.0,1.0,1.0, 83.0/)
      ! A(10,1:N+1)= (/1.0,5.0,4.0,1.0,3.0,6.0,2.0,2.0,2.0,7.0, 155.0/)

      IF (me == root) THEN
        argc = IARGC()  ! IARGC() == NARGS()-1
        IF (argc < 1) THEN
          CALL ExitProg("Input data file missing", me)
        END IF
        CALL GETARG(1, buffer, l)
        fname = buffer(1:l)

        OPEN (UNIT=10, FILE=fname, STATUS="OLD", IOSTAT=xstat)
        IF (xstat > 0) THEN
          CALL ExitProg("Cannot open input file", me)
        END IF

        ! Read size
        WRITE(*,*) "Input data matrix: ",fname
        READ(10,*) N
      END IF
      CALL MPI_BCAST(N, 1, MPI_INTEGER, root, MPI_COMM_WORLD, ierr)
      WRITE(*,'(I2,"]  Size: ",I5)') me,N

      ALLOCATE (A(N,N+1),STAT=xstat)
      IF (xstat /= 0) THEN
        CALL ExitProg("Not enough memory", me)
      END IF
      ALLOCATE (X(N),STAT=xstat)
      IF (xstat /= 0) THEN
        CALL ExitProg("Not enough memory", me)
      END IF

      IF (me == root) THEN
        ! Read matrix
        DO i = 1,N
          READ(10,*, IOSTAT=xstat, ERR=100, END=100) &
            (A(i,j),j=1,N+1)
        END DO
        CLOSE(10)
        WRITE(*,*) "Read matrix OK!"
      END IF
      l = SIZE(A)
      CALL MPI_BCAST(A, l, MPI_DOUBLE_PRECISION, root, MPI_COMM_WORLD, ierr)

      ! Elimination
      WRITE(*,'(I2,"]  Start Elimination")') me
      CALL DATE_AND_TIME(date, xtime, zone, values)
      stime(1:4) = values(5:8)

      CALL GaussianElimination(A, N, X, singular, MPI_COMM_WORLD, nprocs)

      CALL DATE_AND_TIME(date, xtime, zone, values)
      ftime(1:4) = values(5:8)
      CALL ElapsedTime(stime, ftime, etime)

      IF (.NOT. singular) THEN
        IF (me == root) THEN
          WRITE(*,*) "Solution is:"
          DO j = 1, N
            WRITE(*, '(1X, "X(", I4, ") =", F15.6)') j, X(j)
          END DO
          WRITE(*,*)
        END IF
        WRITE(*,'(I2,"]  Start time (main loop):   ",I2.2,":",I2.2,":",I2.2,".",I3.3)') &
              me,stime(1),stime(2),stime(3),stime(4)
        WRITE(*,'(I2,"]  End time (main loop):     ",I2.2,":",I2.2,":",I2.2".",I3.3)') &
              me,ftime(1),ftime(2),ftime(3),ftime(4)
        WRITE(*,'(I2,"]  Elapsed time (main loop): ",I2.2,":",I2.2,":",I2.2".",I3.3)') &
              me,etime(1),etime(2),etime(3),etime(4)
      ELSE
        IF (me == root) THEN
          WRITE(*,*) "Matrix is (nearly) singular" 
        END IF
        !WRITE(*,'(I2,"]  ")') me,"Matrix is (nearly) singular"
      END IF
      CALL MPI_FINALIZE(ierr)
      CALL EXIT(0)

100   CALL ExitProg("Read matrix", me)

CONTAINS

  SUBROUTINE GaussianElimination(A, N, X, singular, comm, nprocs)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT) :: singular
    INTEGER, INTENT(IN) :: N
    INTEGER, INTENT(IN) :: comm, nprocs
    !REAL(KIND=Prec), DIMENSION(N, N+1), INTENT(INOUT) :: A
    !REAL(KIND=Prec), DIMENSION(N), INTENT(OUT) :: X
    !REAL(KIND=Prec), DIMENSION(N+1) :: temp
    !REAL(KIND=Prec) :: abspivot, mult
    !REAL(KIND=Prec), PARAMETER :: epsilon = 1E-13
    DOUBLE PRECISION, DIMENSION(N, N+1), INTENT(INOUT) :: A
    DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: X
    DOUBLE PRECISION, DIMENSION(N+1) :: temp
    DOUBLE PRECISION :: abspivot, mult
    DOUBLE PRECISION, PARAMETER :: epsilon = 1E-13
    INTEGER :: pivotrow
    INTEGER :: bsize, lb, ub
    INTEGER :: lbx, b1, b2, tag, src, dst
    INTEGER :: i, j, k, l
    INTEGER :: me, ierr, root=0
    INTEGER :: status(MPI_STATUS_SIZE)

    CALL MPI_COMM_RANK(comm, me, ierr)

    ! Work distribution - BLOCK
    !bsize = INT(N / nprocs)
    !IF (MOD(N,nprocs) > 1) bsize = bsize + 1
    bsize = CEILING(REAL(N)/REAL(nprocs))
    lb = me*bsize+1
    ub = MIN((me+1)*bsize,N)
    WRITE(*,'(I2,"]  bsize=",I4,"  lb=",I4,"  ub=",I4)') &
          me,bsize,lb,ub

    singular = .FALSE.

    DO k = 1,N

      ! Locate pivot element
      abspivot = ABS(A(k,k))
      pivotrow = k
      DO i = k+1, N
        IF (ABS(A(i,k)) > abspivot) THEN
          abspivot = ABS(A(i,k))
          pivotrow = i
        END IF
      END DO
  
      ! Check if matrix is (nearly) singular
      IF (abspivot < epsilon) THEN
        singular = .TRUE.
        RETURN
      END IF
  
      ! Interchange rows pivotrow and k if necessary
      IF (pivotrow /= k) THEN
        temp(k:N+1) = A(k,k:N+1)
        A(k,k:N+1) = A(pivotrow,k:N+1)
        A(pivotrow,k:N+1) = temp(k:N+1)
      END IF

      lbx = MAX(lb,(k+1))
      IF (MOD(k,200) == 0) THEN
        WRITE(*,'(I2,"] ",I4,"  lbx=",I4,"  ub=",I4)') me,k,lbx,ub
      END IF

      ! Eliminate kth unknown from equations k+1,...,N
      DO i = lbx,ub    !DO i = k+1,N
        mult = A(i,k)/A(k,k)
        !DO j = k+1,N+1
        !  A(i,j) = A(i,j) - mult * A(k,j)
        !END DO
        A(i,k+1:N+1) = A(i,k+1:N+1) - mult*A(k,k+1:N+1)
      END DO
      CALL MPI_BARRIER(comm, ierr)
      DO dst=0,nprocs-1
        IF (dst /= me) THEN
          IF (lbx <= ub) THEN
            tag = 100+me
            l = SIZE(A(lbx:ub,k+1:N+1))
            CALL MPI_SEND(A(lbx:ub,k+1:N+1), l, MPI_DOUBLE_PRECISION, dst, tag, &
                          comm, status, ierr)
          END IF
        ELSE
          !src = dst
          !DO i = 1,nprocs-1
          !  src = MOD(src+1,nprocs)
          DO src = 0,nprocs-1
            IF (src == dst) CYCLE
            b1 = MAX(src*bsize+1,(k+1))
            b2 = MIN((src+1)*bsize,N)
            IF (b1 <= b2) THEN
              tag = 100+src
              l = SIZE(A(b1:b2,k+1:N+1))
              CALL MPI_RECV(A(b1:b2,k+1:N+1), l, MPI_DOUBLE_PRECISION, src, tag, &
                            comm, status, ierr)
            END IF
          END DO 
        END IF
      END DO 

    END DO
    !IF (me == root) THEN
    !  WRITE(*,*) "After main loop"
    !  DO i = 1,N
    !    WRITE(*,'(10F10.4)') (A(i,j),j=1,N+1)
    !  END DO
    !  WRITE(*,*)
    !END IF

    ! Solve X(N-1), X(N-2), ...,X(1) by backward substitution

    ! First calculate X(N)
    X(N) = A(N,N+1) / A(N,N)

    DO j = N-1,1,-1
      !DO i = 1,j
      !  A(i,N+1) = A(i,N+1) - A(i,j+1) * X(j+1)
      !END DO
      !X(j) = A(j,N+1) / A(j,j)
      X(j) = (A(j,N+1) - SUM(A(j,j+1:N) * X(j+1:N))) / A(j,j)
    END DO

    RETURN

  END SUBROUTINE GaussianElimination

  SUBROUTINE ExitProg(message, me)
    CHARACTER(LEN=*), INTENT(IN) :: message
    INTEGER, INTENT(IN) :: me
    INTEGER :: ierr

    WRITE(*,'(I2,"]  Error: ")') me,message
    !CALL MPI_ABORT(1, ierr)
    CALL MPI_FINALIZE(ierr)
    CALL EXIT(1)
      
  END SUBROUTINE ExitProg

  SUBROUTINE ElapsedTime(time0, time1, etime)
    INTEGER, INTENT(IN) :: time0(4)   ! Start time
    INTEGER, INTENT(IN) :: time1(4)   ! End time
    INTEGER, INTENT(OUT) :: etime(4)  ! Elapsed time
    INTEGER :: h, m, s, et
    h = time1(1)
    m = time1(2)
    s = time1(3)
    ! seconds/1000
    et = time1(4) - time0(4)
    IF (et < 0) THEN
      et = et + 1000
      s = s - 1
    END IF
    etime(4) = et
    ! seconds
    et = s - time0(3)
    IF (et < 0) THEN
      et = et + 60
      m = m - 1
    END IF
    etime(3) = et 
    ! minutes
    et = m - time0(2)
    IF (et < 0) THEN
      et = et + 60
      h = h - 1
    END IF
    etime(2) = et
    ! hours
    etime(1) = h - time0(1)

  END SUBROUTINE ElapsedTime

END PROGRAM GAUSS

