      PROGRAM GAUSS
      ! Solving linear equation system  Ax = b
      ! A : Coefficient matrix with dimension (N,N+1)
      ! Right hand side vector of linear equations is stored
      !   in last (N+1)-th column of matrix A
      ! X : Vector of unknowns
      ! N : Number of linear equations

      IMPLICIT NONE
      
      INTEGER, PARAMETER :: Prec = SELECTED_REAL_KIND(15)
      ! SELECTED_REAL_KIND(15) = 8
      ! REAL(KIND=8) <=> DOUBLE PRECISION
      !REAL(KIND=Prec), DIMENSION(:,:), ALLOCATABLE :: A
      !REAL(KIND=Prec), DIMENSION(:), ALLOCATABLE :: X
      !REAL(KIND=Prec), DIMENSION(:), ALLOCATABLE :: temp
      !REAL(KIND=Prec) :: abspivot, mult
      !REAL(KIND=Prec), PARAMETER :: epsilon = 1E-13
      DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: A
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: X
      INTEGER :: N
      LOGICAL :: singular = .FALSE.
      INTEGER :: i, j, xstat
      !CHARACTER*15 :: fname="matrix.dat"
      CHARACTER*15 :: fname
      CHARACTER*70 :: buffer
      INTEGER :: argc
      INTEGER :: stime(4), ftime(4), etime(4), values(8)
      CHARACTER*8 :: date    ! YYYYMMDD
      CHARACTER*10 :: xtime  ! HHMMSS.SSS
      CHARACTER*5 :: zone    ! HHMM

      !argc = NARGS()
      argc = IARGC()  ! IARGC() == NARGS()-1
      IF (argc < 1) THEN
        CALL ExitProg("Input data matrix missing")
      END IF
      CALL GETARG(1, buffer, i)
      !READ(buffer,*) fname
      fname = buffer(1:i)

      OPEN (UNIT=10, FILE=fname, STATUS="OLD", IOSTAT=xstat)
      IF (xstat > 0) THEN
        CALL ExitProg("Cannot open the input file")
      END IF
      ! Read size
      WRITE(*,*) "Input data matrix: ",fname
      READ(10,*) N
      WRITE(*,'(" Size: ",I5)') N
      ALLOCATE (A(N,N+1),STAT=xstat)
      IF (xstat /= 0) THEN
        CALL ExitProg("Not enough memory")
      END IF
      ALLOCATE (X(N),STAT=xstat)
      IF (xstat /= 0) THEN
      END IF

      ! Read matrix
      DO i = 1,N
        READ(10,*, IOSTAT=xstat, ERR=100, END=100) &
          (A(i,j),j=1,N+1)
      END DO
      CLOSE(10)
      WRITE(*,*) "Read matrix OK!"

      ! Elimination
      WRITE(*,*) "Start Elimination"
      CALL DATE_AND_TIME(date, xtime, zone, values)
      stime(1:4) = values(5:8)

      CALL GaussianElimination(A, N, X, singular)

      CALL DATE_AND_TIME(date, xtime, zone, values)
      ftime(1:4) = values(5:8)

      IF (.NOT. singular) THEN

        WRITE(*,*) "Solution is:"
        DO j = 1, N
          WRITE(*, '(1X, "X(", I4, ") =", F15.6)') j, X(j)
        END DO
        WRITE(*,*)
        CALL ElapsedTime(stime, ftime, etime)
        WRITE(*,'("Start time (main loop):   ",I2.2,":",I2.2,":",I2.2,".",I3.3)') &
              stime(1),stime(2),stime(3),stime(4)
        WRITE(*,'("End time (main loop):     ",I2.2,":",I2.2,":",I2.2,".",I3.3)') &
              ftime(1),ftime(2),ftime(3),ftime(4)
        WRITE(*,'("Elapsed time (main loop): ",I2.2,":",I2.2,":",I2.2,".",I3.3)') &
              etime(1),etime(2),etime(3),etime(4)

      ELSE
        WRITE(*,*) "Matrix is (nearly) singular" 
      END IF
      CALL EXIT(0)

100   CALL ExitProg("Read input data matrix")

CONTAINS

  SUBROUTINE GaussianElimination(A, N, X, singular)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT) :: singular
    INTEGER, INTENT(IN) :: N      
    !REAL(KIND=Prec), DIMENSION(N, N+1), INTENT(INOUT) :: A
    !REAL(KIND=Prec), DIMENSION(N), INTENT(OUT) :: X
    !REAL(KIND=Prec), DIMENSION(N+1) :: temp
    !REAL(KIND=Prec) :: abspivot, mult
    !REAL(KIND=Prec), PARAMETER :: epsilon = 1E-13
    DOUBLE PRECISION, DIMENSION(N, N+1), INTENT(INOUT) :: A
    DOUBLE PRECISION, DIMENSION(N), INTENT(OUT) :: X
    DOUBLE PRECISION, DIMENSION(N+1) :: temp
    DOUBLE PRECISION :: abspivot, mult
    DOUBLE PRECISION, PARAMETER :: epsilon = 1E-13
    INTEGER :: pivotrow, i, j, k

    singular = .FALSE.
    DO k = 1,N

      ! Locate pivot element
      abspivot = ABS(A(k,k))
      pivotrow = k
      DO i = k+1, N
        IF (ABS(A(i,k)) > abspivot) THEN
          abspivot = ABS(A(i,k))
          pivotrow = i
        END IF
      END DO
  
      ! Check if matrix is (nearly) singular
      IF (abspivot < epsilon) THEN
        singular = .TRUE.
        RETURN
      END IF
  
      ! Interchange rows pivotrow and i if necessary
      IF (pivotrow /= k) THEN
        temp(k:N+1) = A(k,k:N+1)
        A(k,k:N+1) = A(pivotrow,k:N+1)
        A(pivotrow,k:N+1) = temp(k:N+1)
      END IF

      ! Eliminate kth unknown from equations k+1,...,N
      DO i = k+1,N
        mult = A(i,k)/A(k,k)
        !DO j = k+1,N+1
        !  A(i,j) = A(i,j) - mult * A(k,j)
        !END DO
        A(i,k+1:N+1) = A(i,k+1:N+1) - mult*A(k,k+1:N+1)
      END DO
    END DO

    ! Solve X(N-1), X(N-2), ...,X(1) by backward substitution

    ! First calculate X(N)
    X(N) = A(N,N+1) / A(N,N)
    DO j = N-1,1,-1
      !DO i = 1,j
      !  A(i,N+1) = A(i,N+1) - A(i,j+1) * X(j+1)
      !END DO
      !X(j) = A(j,N+1) / A(j,j)
      X(j) = (A(j,N+1) - SUM(A(j,j+1:N) * X(j+1:N))) / A(j,j)
    END DO
    RETURN

  END SUBROUTINE GaussianElimination

  SUBROUTINE ExitProg(message)
    CHARACTER(LEN=*), INTENT(IN) :: message

    WRITE(*,*) message
    CALL EXIT(1)

  END SUBROUTINE ExitProg

  SUBROUTINE ElapsedTime(time0, time1, etime)
    INTEGER, INTENT(IN) :: time0(4)   ! Start time
    INTEGER, INTENT(IN) :: time1(4)   ! End time
    INTEGER, INTENT(OUT) :: etime(4)  ! Elapsed time
    INTEGER :: h, m, s, et
    h = time1(1)
    m = time1(2)
    s = time1(3)
    ! seconds/1000
    et = time1(4) - time0(4)
    IF (et < 0) THEN
      et = et + 1000
      s = s - 1
    END IF
    etime(4) = et
    ! seconds
    et = s - time0(3)
    IF (et < 0) THEN
      et = et + 60
      m = m - 1
    END IF
    etime(3) = et 
    ! minutes
    et = m - time0(2)
    IF (et < 0) THEN
      et = et + 60
      h = h - 1
    END IF
    etime(2) = et
    ! hours
    etime(1) = h - time0(1)

  END SUBROUTINE ElapsedTime

END PROGRAM GAUSS

