#!/bin/sh
#
# Return values:
# 0   - Success
# 1   - Failure

# Input parameters
###################
numarg=$#
if [ $numarg -ne 3 ]; then
  echo " [ERROR]: Incorrect number of arguments ($numarg)!"
  echo "Usage: <ncpus> <exec_file> <output_arg>"
  exit 1
fi
NCPUS=$1
EXEC=$2
OARG=$3
echo "Arguments:"
echo "  NCPUS= $NCPUS"
echo "  EXEC= $EXEC"
echo "  OARG= $OARG"

# Print an error message and exit
function error_msg () {
  echo "JACOBI [ERROR]:" $@
  exit 1
}

# Print an info message
function info_msg () {
  echo "JACOBI [INFO]:" $@
}
echo "JACOBI [INFO]: Running start ("`date`")"

WORKDIR=`pwd`
ls -l
[ -e "$EXEC" ] || error_msg "File $EXEC not available!"
chmod 755 $EXEC

#==================================================================
# Setup for mpirun/mpiexec
#
#MPI_FLAVOUR=OPENMPI
#if [ "x$MPI_OPENMPI_PATH" == "x" ]; then
#  info_msg "Path MPI_OPENMPI_PATH not set!"
## On ce2.ui.savba.sk
#  MPI_OPENMPI_PATH="/opt/openmpi-1.3.2"
#fi
#info_msg "MPI_OPENMPI_PATH: $MPI_OPENMPI_PATH"
#MPI_PREFIX="--prefix ${MPI_OPENMPI_PATH}"
#MPI_EXEC=${MPI_OPENMPI_PATH}/bin/mpiexec
#
MPI_FLAVOUR=MPICH
if [ "x$MPI_MPICH_PATH" == "x" ]; then
  info_msg "Path MPI_MPICH_PATH not set!"
# On ce2.ui.savba.sk
  MPI_MPICH_PATH="/opt/mpich-1.2.7p1"
fi
info_msg "MPI_MPICH_PATH: $MPI_MPICH_PATH"
MPI_PREFIX=""
MPI_EXEC=${MPI_MPICH_PATH}/bin/mpirun
#
[ -e "${MPI_EXEC}" ] || error_msg "File ${MPI_EXEC} not available!"
MPI_PARAMS="-np $NCPUS"
MPI_COMMAND="${MPI_EXEC} ${MPI_PREFIX} ${MPI_PARAMS}"
#
#==================================================================
info_msg "Execution:"
echo "    ${MPI_COMMAND} $EXEC 1>$OARG 2>&1"
${MPI_COMMAND} $EXEC 1>$OARG 2>&1
excode=$?
if [ $excode -ne 0 ]; then
  error_msg "Running failed!"
fi

echo "JACOBI [INFO]: Running finish ("`date`")"

exit $excode

