#!/bin/sh
#
# Return values:
# 0   - Success
# 1   - Failure

# Input parameters
###################
numarg=$#
if [ $numarg -ne 4 ]; then
  echo "MAXCUT [ERROR]: Incorrect number of arguments ($numarg)!"
  echo "Usage: <ncpus> <exec_file> <input_arg> <output_arg>"
  exit 1
fi
NCPUS=$1
EXEC=$2
IARG=$3
OARG=$4
echo "Arguments:"
echo "  NCPUS= $NCPUS"
echo "  EXEC= $EXEC"
echo "  IARG= $IARG"
echo "  OARG= $OARG"

# Print an error message and exit
function error_msg () {
  echo "MAXCUT [ERROR]:" $@
  exit 1
}

# Print an info message
function info_msg () {
  echo "MAXCUT [INFO]:" $@
}
echo "MAXCUT [INFO]: Running start ("`date`")"

WORKDIR=`pwd`
ls -l
[ -e "$EXEC" ] || error_msg "File $EXEC not available!"
[ -e "$IARG" ] || error_msg "File $IARG not available!"
chmod 755 $EXEC

#==================================================================
# Setup for mpiexec
#
MPI_FLAVOUR=OPENMPI
if [ "x$MPI_OPENMPI_PATH" == "x" ]; then
  info_msg "Path MPI_OPENMPI_PATH not set!"
# On ce2.ui.savba.sk
  MPI_OPENMPI_PATH="/opt/openmpi-1.3.2"
fi
info_msg "MPI_OPENMPI_PATH: $MPI_OPENMPI_PATH"
#MPI_PREFIX="--prefix ${MPI_OPENMPI_PATH}"
MPI_PREFIX=""
MPI_EXEC=${MPI_OPENMPI_PATH}/bin/mpiexec
[ -e "${MPI_EXEC}" ] || error_msg "File ${MPI_EXEC} not available!"
#
#MPI_PARAMS="-wdir ${WORKDIR} -np $NCPUS"
MPI_PARAMS="-np $NCPUS"
MPI_COMMAND="${MPI_EXEC} ${MPI_PREFIX} ${MPI_PARAMS}"
#
#==================================================================
info_msg "Execution:"
echo "  ${MPI_COMMAND} $EXEC $IARG 1>$OARG 2>&1"
${MPI_COMMAND} $EXEC $IARG 1>$OARG 2>&1
excode=$?
if [ $excode -ne 0 ]; then
  error_msg "Running failed!"
fi

echo "MAXCUT [INFO]: Running finish ("`date`")"

exit $excode

